#!/usr/bin/env python3

OP_FALSE = 0x00
OP_TRUE = 0x51

OP_0 = 0x00
OP_1 = 0x51
OP_2 = 0x52
OP_3 = 0x53
OP_4 = 0x54
OP_5 = 0x55
OP_6 = 0x56
OP_7 = 0x57
OP_8 = 0x58
OP_9 = 0x59
OP_10 = 0x5a
OP_11 = 0x5b
OP_12 = 0x5c
OP_13 = 0x5d
OP_14 = 0x5e
OP_15 = 0x5f
OP_16 = 0x60

OP_DATA_1 = 0x01
OP_DATA_2 = 0x02
OP_DATA_3 = 0x03
OP_DATA_4 = 0x04
OP_DATA_5 = 0x05
OP_DATA_6 = 0x06
OP_DATA_7 = 0x07
OP_DATA_8 = 0x08
OP_DATA_9 = 0x09
OP_DATA_10 = 0x0a
OP_DATA_11 = 0x0b
OP_DATA_12 = 0x0c
OP_DATA_13 = 0x0d
OP_DATA_14 = 0x0e
OP_DATA_15 = 0x0f
OP_DATA_16 = 0x10
OP_DATA_17 = 0x11
OP_DATA_18 = 0x12
OP_DATA_19 = 0x13
OP_DATA_20 = 0x14
OP_DATA_21 = 0x15
OP_DATA_22 = 0x16
OP_DATA_23 = 0x17
OP_DATA_24 = 0x18
OP_DATA_25 = 0x19
OP_DATA_26 = 0x1a
OP_DATA_27 = 0x1b
OP_DATA_28 = 0x1c
OP_DATA_29 = 0x1d
OP_DATA_30 = 0x1e
OP_DATA_31 = 0x1f
OP_DATA_32 = 0x20
OP_DATA_33 = 0x21
OP_DATA_34 = 0x22
OP_DATA_35 = 0x23
OP_DATA_36 = 0x24
OP_DATA_37 = 0x25
OP_DATA_38 = 0x26
OP_DATA_39 = 0x27
OP_DATA_40 = 0x28
OP_DATA_41 = 0x29
OP_DATA_42 = 0x2a
OP_DATA_43 = 0x2b
OP_DATA_44 = 0x2c
OP_DATA_45 = 0x2d
OP_DATA_46 = 0x2e
OP_DATA_47 = 0x2f
OP_DATA_48 = 0x30
OP_DATA_49 = 0x31
OP_DATA_50 = 0x32
OP_DATA_51 = 0x33
OP_DATA_52 = 0x34
OP_DATA_53 = 0x35
OP_DATA_54 = 0x36
OP_DATA_55 = 0x37
OP_DATA_56 = 0x38
OP_DATA_57 = 0x39
OP_DATA_58 = 0x3a
OP_DATA_59 = 0x3b
OP_DATA_60 = 0x3c
OP_DATA_61 = 0x3d
OP_DATA_62 = 0x3e
OP_DATA_63 = 0x3f
OP_DATA_64 = 0x40
OP_DATA_65 = 0x41
OP_DATA_66 = 0x42
OP_DATA_67 = 0x43
OP_DATA_68 = 0x44
OP_DATA_69 = 0x45
OP_DATA_70 = 0x46
OP_DATA_71 = 0x47
OP_DATA_72 = 0x48
OP_DATA_73 = 0x49
OP_DATA_74 = 0x4a
OP_DATA_75 = 0x4b

OP_PUSHDATA1 = 0x4c
OP_PUSHDATA2 = 0x4d
OP_PUSHDATA4 = 0x4e
OP_1NEGATE = 0x4f
OP_NOP = 0x61

OP_JUMP = 0x63
OP_JUMPIF = 0x64
OP_VERIFY = 0x69
OP_FAIL = 0x6a
OP_CHECKPREDICATE = 0xc0

OP_TOALTSTACK = 0x6b
OP_FROMALTSTACK = 0x6c
OP_2DROP = 0x6d
OP_2DUP = 0x6e
OP_3DUP = 0x6f
OP_2OVER = 0x70
OP_2ROT = 0x71
OP_2SWAP = 0x72
OP_IFDUP = 0x73
OP_DEPTH = 0x74
OP_DROP = 0x75
OP_DUP = 0x76
OP_NIP = 0x77
OP_OVER = 0x78
OP_PICK = 0x79
OP_ROLL = 0x7a
OP_ROT = 0x7b
OP_SWAP = 0x7c
OP_TUCK = 0x7d

OP_CAT = 0x7e
OP_SUBSTR = 0x7f
OP_LEFT = 0x80
OP_RIGHT = 0x81
OP_SIZE = 0x82
OP_CATPUSHDATA = 0x89

OP_INVERT = 0x83
OP_AND = 0x84
OP_OR = 0x85
OP_XOR = 0x86
OP_EQUAL = 0x87
OP_EQUALVERIFY = 0x88

OP_1ADD = 0x8b
OP_1SUB = 0x8c
OP_2MUL = 0x8d
OP_2DIV = 0x8e
OP_NEGATE = 0x8f
OP_ABS = 0x90
OP_NOT = 0x91
OP_0NOTEQUAL = 0x92
OP_ADD = 0x93
OP_SUB = 0x94
OP_MUL = 0x95
OP_DIV = 0x96
OP_MOD = 0x97
OP_LSHIFT = 0x98
OP_RSHIFT = 0x99
OP_BOOLAND = 0x9a
OP_BOOLOR = 0x9b
OP_NUMEQUAL = 0x9c
OP_NUMEQUALVERIFY = 0x9d
OP_NUMNOTEQUAL = 0x9e
OP_LESSTHAN = 0x9f
OP_GREATERTHAN = 0xa0
OP_LESSTHANOREQUAL = 0xa1
OP_GREATERTHANOREQUAL = 0xa2
OP_MIN = 0xa3
OP_MAX = 0xa4
OP_WITHIN = 0xa5

OP_SHA256 = 0xa8
OP_SHA3 = 0xaa
OP_HASH160 = 0xab
OP_CHECKSIG = 0xac
OP_CHECKMULTISIG = 0xad
OP_TXSIGHASH = 0xae

OP_CHECKOUTPUT = 0xc1
OP_ASSET = 0xc2
OP_AMOUNT = 0xc3
OP_PROGRAM = 0xc4
OP_INDEX = 0xc9
OP_ENTRYID = 0xca
OP_OUTPUTID = 0xcb
OP_BLOCKHEIGHT = 0xcd


# OP_DATAS
def op_datas():
    return [
        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d,
        0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a,
        0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,
        0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x34,
        0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f, 0x40, 0x41,
        0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a, 0x4b
    ]



#!/usr/bin/env python3


# Bytom configuration
def config():

    return {
        "mainnet": {
            "bytom": "http://localhost:9888",
            "blockmeta": "https://blockmeta.com/api/v3",
            "blockcenter": {
                "v2": "https://bcapi.bystack.com/api/v2/btm",
                "v3": "https://bcapi.bystack.com/bytom/v3"
            },
            "mov": "https://ex.movapi.com/bytom/v3"
        },
        "solonet": {
            "bytom": "http://localhost:9888",
            "blockmeta": None,
            "blockcenter": {
                "v2": None,
                "v3": None
            },
            "mov": None
        },
        "testnet": {
            "bytom": "http://localhost:9888",
            "blockmeta": None,
            "blockcenter": {
                "v2": None,
                "v3": None
            },
            "mov": None
        },
        "network": "mainnet",
        "timeout": 60,
        "asset": "ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
        "symbols": {
            "BTM": 1,
            "mBTM": 1000,
            "NEU": 100_000_000
        },
        "fee": 10_000_000,
        "confirmations": 1
    }



#!/usr/bin/env python3

from binascii import unhexlify
from hashlib import new, sha3_256

from ..libs.segwit import encode
from .opcode import (
    OP_DUP, OP_HASH160, OP_SHA3, OP_EQUALVERIFY, OP_TXSIGHASH, OP_SWAP, OP_CHECKPREDICATE, OP_CHECKSIG
)
from .builder import Builder


def public_key_hash(public_key: str) -> str:
    """
    Bytom public key hash.

    :param public_key: Bytom contract program(bytecode).
    :type public_key: str
    :return: hash -- Public key ripemd160 hash.

    >>> from pybytom.script import public_key_hash
    >>> public_key_hash("5b5a06f6fbcb74b58ebb42293808fec6222234df6c97d7c1cff6d857a6024dc2")
    "875240ba66646d900c59dd20d843351c2fcbeedc"
    """

    if not isinstance(public_key, str):
        raise TypeError("invalid public key type, public_key must be string format")

    ripemd160 = new("ripemd160")
    ripemd160.update(unhexlify(public_key))
    public_hash = ripemd160.hexdigest()
    return public_hash


def script_hash(bytecode: str) -> str:
    """
    Bytom smart contract program(bytecode) script hash.

    :param bytecode: Bytom contract program(bytecode).
    :type bytecode: str
    :return: hash -- Script SHA3-256 hash.

    >>> from pybytom.script import script_hash
    >>> script_hash("7baa8800c3c251547ac1")
    "e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3"
    """

    if not isinstance(bytecode, str):
        raise TypeError("invalid bytecode type, bytecode must be string format")

    return sha3_256(unhexlify(bytecode)).hexdigest()


def p2pkh_program(public_key_hash: str) -> str:
    """
    Get P2PKH program return the segwit pay to public key hash program.

    :param public_key_hash: Bytom public key hash.
    :type public_key_hash: str
    :return: program -- Bytom pay to public key hash program.

    >>> from pybytom.script import p2pkh_program
    >>> p2pkh_program("875240ba66646d900c59dd20d843351c2fcbeedc")
    "0014875240ba66646d900c59dd20d843351c2fcbeedc"
    """

    if not isinstance(public_key_hash, str):
        raise TypeError("invalid public key hash type, public_key_hash must be string format")

    builder = Builder()
    builder.add_op(OP_DUP)
    builder.add_op(OP_HASH160)
    builder.add_bytes(unhexlify(public_key_hash))
    builder.add_op(OP_EQUALVERIFY)
    builder.add_op(OP_TXSIGHASH)
    builder.add_op(OP_SWAP)
    builder.add_op(OP_CHECKSIG)
    return builder.hex_digest()


def p2wpkh_program(public_key_hash: str) -> str:
    """
    Get P2WPKH program return the segwit pay to witness public key hash program.

    :param public_key_hash: Bytom public key hash.
    :type public_key_hash: str
    :return: program -- Bytom Public key hash program.

    >>> from pybytom.script import p2wpkh_program
    >>> p2wpkh_program("875240ba66646d900c59dd20d843351c2fcbeedc")
    "0014875240ba66646d900c59dd20d843351c2fcbeedc"
    """

    if not isinstance(public_key_hash, str):
        raise TypeError("invalid public key hash type, public_key_hash must be string format")

    builder = Builder()
    builder.add_int(0)
    builder.add_bytes(unhexlify(public_key_hash))
    return builder.hex_digest()


def p2wpkh_address(public_key_hash: str, network="solonet") -> str:
    """
    Get P2WPKH address return the segwit pay to public key hash address.

    :param public_key_hash: Bytom public key hash.
    :type public_key_hash: str
    :param network: Bytom network, default to solonet.
    :type network: str
    :return: address -- Bytom pay to public key hash address.

    >>> from pybytom.script import p2wpkh_address
    >>> p2wpkh_address("875240ba66646d900c59dd20d843351c2fcbeedc")
    "bm1qsafypwnxv3keqrzem5sdsse4rshuhmku7kpnxq"
    """

    if not isinstance(public_key_hash, str):
        raise TypeError("invalid public key hash type, public_key_hash must be string format")
    if not isinstance(network, str):
        raise TypeError("invalid network type, network must be string format")

    if len(unhexlify(public_key_hash)) != 20:
        raise ValueError("invalid script hash, witness program must be 20 bytes for p2wpkh")

    if network not in "mainnet/solonet/testnet".split("/"):
        raise ValueError("invalid network option, choose only mainnet, solonet and testnet network")
    elif network == "mainnet":
        return encode("bm", 0x00, unhexlify(public_key_hash))
    elif network == "solonet":
        return encode("sm", 0x00, unhexlify(public_key_hash))
    elif network == "testnet":
        return encode("tm", 0x00, unhexlify(public_key_hash))


def p2sh_program(script_hash: str) -> str:
    """
    Get P2WSH program return the segwit pay to script hash program.

    :param script_hash: Bytom contract program(bytecode) script hash.
    :type script_hash: str
    :return: program -- Bytom pay to script hash program.

    >>> from pybytom.script import p2sh_program
    >>> p2sh_program("e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3")
    "0020e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3"
    """

    if not isinstance(script_hash, str):
        raise TypeError("invalid script hash type, script_hash must be string format")

    builder = Builder()
    builder.add_op(OP_DUP)
    builder.add_op(OP_SHA3)
    builder.add_bytes(unhexlify(script_hash))
    builder.add_op(OP_EQUALVERIFY)
    builder.add_int(-1)
    builder.add_op(OP_SWAP)
    builder.add_int(0)
    builder.add_op(OP_CHECKPREDICATE)
    return builder.hex_digest()


def p2wsh_program(script_hash: str) -> str:
    """
    Get P2WSH program return the segwit pay to script hash program.

    :param script_hash: Bytom contract program(bytecode) script hash.
    :type script_hash: str
    :return: program -- Bytom script hash program.

    >>> from pybytom.script import p2wsh_program
    >>> p2wsh_program("e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3")
    "0020e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3"
    """

    if not isinstance(script_hash, str):
        raise TypeError("invalid script hash type, script_hash must be string format")

    builder = Builder()
    builder.add_int(0)
    builder.add_bytes(unhexlify(script_hash))
    return builder.hex_digest()


def p2wsh_address(script_hash: str, network="solonet") -> str:
    """
    Get P2WSH address return the segwit pay to script hash address.

    :param script_hash: Bytom contract program(bytecode) script hash.
    :type script_hash: str
    :param network: Bytom network, default to solonet.
    :type network: str
    :return: address -- Bytom pay to script hash address.

    >>> from pybytom.script import p2wsh_address
    >>> p2wsh_address("e47eaf5e3a7898197068e2dd7c0209331b36a1b7a73aeb8090df816a3ce5b7d3")
    "bm1qu3l27h360zvpjurgutwhcqsfxvdndgdh5uawhqysm7qk5089klfsrrlhez"
    """

    if not isinstance(script_hash, str):
        raise TypeError("invalid script hash type, script_hash must be string format")
    if not isinstance(network, str):
        raise TypeError("invalid network type, network must be string format")

    if len(unhexlify(script_hash)) != 32:
        raise ValueError("invalid script hash, witness program must be 32 bytes for p2wsh")

    if network not in "mainnet/solonet/testnet".split("/"):
        raise ValueError("invalid network option, choose only mainnet, solonet and testnet network")
    elif network == "mainnet":
        return encode("bm", 0x00, unhexlify(script_hash))
    elif network == "solonet":
        return encode("sm", 0x00, unhexlify(script_hash))
    elif network == "testnet":
        return encode("tm", 0x00, unhexlify(script_hash))



#!/usr/bin/env python3

from binascii import hexlify
from mnemonic.mnemonic import Mnemonic
from typing import Optional, Union

import os

from .libs.segwit import decode
from .exceptions import SymbolError
from .config import config

# Bytom config
config = config()


def generate_entropy(strength=128) -> str:
    """
    Generate entropy hex string.

    :param strength: Entropy strength, default to 128.
    :type strength: int
    :returns: str -- Entropy hex string.

    >>> from pybytom.utils import generate_entropy
    >>> generate_entropy(strength=128)
    "ee535b143b0d9d1f87546f9df0d06b1a"
    """

    if strength not in [128, 160, 192, 224, 256]:
        raise ValueError(
            "Strength should be one of the following "
            "[128, 160, 192, 224, 256], but it is not (%d)."
            % strength
        )
    return hexlify(os.urandom(strength // 8)).decode()


def generate_mnemonic(language="english", strength=128) -> str:
    """
    Generate 12 word mnemonic.

    :param language: Mnemonic language, default to english.
    :type language: str
    :param strength: Entropy strength, default to 128.
    :type strength: int
    :returns: str -- 12 word mnemonic.

    >>> from pybytom.utils import generate_mnemonic
    >>> generate_mnemonic(language="french")
    "sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure"
    """

    if language and language not in ["english", "french", "italian", "japanese",
                                     "chinese_simplified", "chinese_traditional", "korean", "spanish"]:
        raise ValueError("invalid language, use only this options english, french, "
                         "italian, spanish, chinese_simplified, chinese_traditional, japanese or korean languages.")
    if strength not in [128, 160, 192, 224, 256]:
        raise ValueError(
            "Strength should be one of the following "
            "[128, 160, 192, 224, 256], but it is not (%d)."
            % strength
        )

    return Mnemonic(language=language).generate(strength=strength)


def is_mnemonic(mnemonic, language=None) -> bool:
    """
    Check 12 word mnemonic is Valid.

    :param mnemonic: 12 word mnemonic.
    :type mnemonic: str
    :param language: Mnemonic language, default to None.
    :type language: str
    :returns: bool -- True/False.

    >>> from pybytom.utils import is_mnemonic
    >>> is_mnemonic("sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure")
    True
    """

    if language and language not in ["english", "french", "italian", "japanese",
                                     "chinese_simplified", "chinese_traditional", "korean", "spanish"]:
        raise ValueError("invalid language, use only this options english, french, "
                         "italian, spanish, chinese_simplified, chinese_traditional, japanese or korean languages.")
    try:
        if language is None:
            for _language in ["english", "french", "italian",
                              "chinese_simplified", "chinese_traditional", "japanese", "korean", "spanish"]:
                valid = False
                if Mnemonic(language=_language).check(mnemonic=mnemonic) is True:
                    valid = True
                    break
            return valid
        else:
            return Mnemonic(language=language).check(mnemonic=mnemonic)
    except:
        return False


def get_mnemonic_language(mnemonic) -> Optional[str]:
    """
    Get mnemonic language.

    :param mnemonic: 12 word mnemonic.
    :type mnemonic: str
    :returns: str -- Mnemonic language.

    >>> from pybytom.utils import get_mnemonic_language
    >>> get_mnemonic_language("sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure")
    "french"
    """

    if not is_mnemonic(mnemonic=mnemonic):
        raise ValueError("invalid 12 word mnemonic.")

    language = None
    for _language in ["english", "french", "italian",
                      "chinese_simplified", "chinese_traditional", "japanese", "korean", "spanish"]:
        if Mnemonic(language=_language).check(mnemonic=mnemonic) is True:
            language = _language
            break
    return language


def amount_converter(amount: float, symbol: str = "NEU2BTM") -> Union[int, float]:
    """
    Amount converter

    :param amount: Bytom amount.
    :type amount: float
    :param symbol: Bytom symbol, default to NEU2BTM
    :type symbol: str
    :returns: float -- BTM asset amount.

    >>> from pybytom.utils import amount_converter
    >>> amount_converter(amount=10_000_000, symbol="NEU2BTM")
    0.1
    """

    if symbol not in ["BTM2mBTM", "BTM2NEU", "mBTM2BTM", "mBTM2NEU", "NEU2BTM", "NEU2mBTM"]:
        raise SymbolError(f"Invalid '{symbol}' symbol/type",
                          "choose only 'BTM2mBTM', 'BTM2NEU', 'mBTM2BTM', 'mBTM2NEU', 'NEU2BTM' or 'NEU2mBTM' symbols.")

    # Constant values
    BTM, mBTM, NEU = (
        config["symbols"]["BTM"],
        config["symbols"]["mBTM"],
        config["symbols"]["NEU"]
    )

    if symbol == "BTM2mBTM":
        return float((amount * mBTM) / BTM)
    elif symbol == "BTM2NEU":
        return int((amount * NEU) / BTM)
    elif symbol == "mBTM2BTM":
        return float((amount * BTM) / mBTM)
    elif symbol == "mBTM2NEU":
        return int((amount * NEU) / mBTM)
    elif symbol == "NEU2BTM":
        return float((amount * BTM) / NEU)
    elif symbol == "NEU2mBTM":
        return int((amount * mBTM) / NEU)


def is_network(network: str) -> bool:
    """
    Check network type.

    :param network: Bytom network.
    :type network: str
    :returns: bool -- Checked network.

    >>> from pybytom.utils import is_network
    >>> is_network("solonet")
    True
    """

    if isinstance(network, str):
        return network.lower() in ["mainnet", "solonet", "testnet"]
    raise TypeError("network must be string format")


def is_address(address, network=None) -> bool:
    """
    Check Bytom address.

    :param address: Bytom address.
    :type address: str
    :param network: Bytom network, defaults to None.
    :type network: str
    :returns: bool -- Bytom valid/invalid address.

    >>> from pybytom.utils import is_address
    >>> is_address("bm1q9ndylx02syfwd7npehfxz4lddhzqsve2fu6vc7", "mainnet")
    True
    """

    if isinstance(address, str):
        if network is None:
            for hrp in ["bm", "sm", "tm"]:
                valid = False
                if address.startswith(hrp) and \
                        not decode(hrp, address) == (None, None):
                    valid = True
                    break
            return valid
        if not isinstance(network, str):
            raise TypeError("network must be string format")
        elif network == "mainnet":
            return address.startswith("bm") and not decode("bm", address) == (None, None)
        elif network == "solonet":
            return address.startswith("sm") and not decode("sm", address) == (None, None)
        elif network == "testnet":
            return address.startswith("tm") and not decode("tm", address) == (None, None)
        else:
            raise ValueError("invalid network, use only this options mainnet, solonet or testnet networks.")
    raise TypeError("address must be string format")


def is_vapor_address(address, network=None) -> bool:
    """
    Check Bytom vapor address.

    :param address: Bytom vapor address.
    :type address: str
    :param network: Bytom network, defaults to None.
    :type network: str
    :returns: bool -- Bytom valid/invalid address.

    >>> from pybytom.utils import is_vapor_address
    >>> is_vapor_address("vp1q9ndylx02syfwd7npehfxz4lddhzqsve2za23ag", "mainnet")
    True
    """

    if isinstance(address, str):
        if network is None:
            for hrp in ["vp", "sp", "tp"]:
                valid = False
                if address.startswith(hrp) and \
                        not decode(hrp, address) == (None, None):
                    valid = True
                    break
            return valid
        if not isinstance(network, str):
            raise TypeError("network must be string format")
        elif network == "mainnet":
            return address.startswith("vp") and not decode("vp", address) == (None, None)
        elif network == "solonet":
            return address.startswith("sp") and not decode("sp", address) == (None, None)
        elif network == "testnet":
            return address.startswith("tp") and not decode("tp", address) == (None, None)
        else:
            raise ValueError("invalid network, use only this options mainnet, solonet or testnet networks.")
    raise TypeError("address must be string format")



#!/usr/bin/env python3

from .script import (
    public_key_hash, script_hash,
    p2pkh_program, p2sh_program,
    p2wpkh_program, p2wsh_program,
    p2wpkh_address, p2wsh_address
)


__all__ = [
    # Hash
    "public_key_hash", "script_hash",
    # Program
    "p2wpkh_program", "p2wsh_program",
    "p2pkh_program", "p2sh_program",
    # Address
    "p2wpkh_address", "p2wsh_address"
]
